<?php
/* Roku SoundBridge Interface Library
 * Version 1.0.0
 * Copyright 2005-2006, Steve Blinch
 * http://code.blitzaffe.com
 * ============================================================================
 *
 * DESCRIPTION
 *
 * Simple class to interface with a Roku SoundBridge M500 network music player
 * (http://www.rokulabs.com/products/soundbridge/) via telnet and HTTP.
 *
 *
 * REQUIREMENTS
 *
 * - You must have a Roku SoundBridge on your network (obviously)
 * - You must have my HTTPRetriever class, available at http://code.blitzaffe.com,
 *   if you want to use the get_state() method
 *
 * STATUS
 *
 * Supports obtaining various information from your SoundBridge (network info,
 * current song, status, etc.), displaying text on its display, and basic
 * control operatins such as start, stop, volume, etc.
 *
 * Basic MMC support is present, but since I personally only use my Soundbridge
 * to connect to my Icecast streaming server on my network, most of the code
 * necessary for working with iTunes and other media libraries has not yet been
 * implemented.
 *
 *
 * EXAMPLE
 *
 * require_once("class_RokuSoundBridge.php");
 * 
 * $ip = "192.168.0.150"; // Your SoundBridge's IP address
 * 
 * $soundbridge = &new RokuSoundBridge();
 * 
 * // get_state() is the only method that can be called without first calling
 * // the connect() method
 * if (!$soundbridge->get_state($ip,8080)) die("Error: ".$soundbridge->error);
 * if ($soundbridge->is_playing) echo "NOW PLAYING! ";
 * if ($soundbridge->is_paused) echo "NOW PAUSED! ";
 * echo "Soundbridge is currently ";
 * if ($soundbridge->track) echo "on track {$soundbridge->track}, and is ";
 * echo $soundbridge->state.".<br>";
 * 
 * // connect to the SoundBridge
 * if (!$soundbridge->connect($ip)) die("Error: ".$soundbridge->error);
 * 
 * // display a message on its LED screen for 2 seconds
 * $soundbridge->display(
 * 	"Displayed via RokuSoundBridge class!",
 * 	"Visit http://code.blitzaffe.com",
 * 	2000000
 * );
 * 
 * echo "SoundBridge has been up and running for ".$soundbridge->uptime()." seconds.<br>";
 * 
 * echo "Process list:<br><pre>" . 
 * 	print_r($soundbridge->process_list(),true) . "</pre><br>";
 * 
 * echo "Network Interfaces:<br><pre>" .
 * 	print_r($soundbridge->network_interfaces()) . "</pre><br>";
 * 
 * $soundbridge->disconnect();
 * 
 *
 * LICENSE
 *
 * This script is free software; you can redistribute it and/or modify it under the
 * terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 *
 * This script is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *	
 * You should have received a copy of the GNU General Public License along
 * with this script; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

require_once("class_HTTPRetriever.php");

class RokuSoundBridge {
	function RokuSoundBridge() {
		$this->fp = false;
	}
	
	// queries the SoundBridge for its current state and track; this is the only
	// method that does not require a call to RokuSoundBridge->connect() first
	function get_state($ip,$port=80) {
		$http = &new HTTPRetriever();
		if (!$http->get("http://{$ip}:{$port}/SoundBridgeNP.html")) {
			$this->error = "Could not get Roku status page: HTTP Error #{$http->result_code}: {$http->result_text}";
			return false;
		}
		
		if (preg_match("/<font.*?>\s*?SoundBridge\s+is\s+(.*?)\.\s*?<\/font>[\s\S]+<font.*?><strong>\s+(.*?)<br>\s*?<\/strong>\s*?(.*?)<br>/",$http->response,$matches)) {
			list(,$state,$track,$station) = $matches;
			$this->state = $state;
			$this->is_playing = ($state=="playing");
			$this->is_paused = ($state=="paused");
			$this->track = $track;
			$this->station = $station;
		} else {
			$this->error = "Unrecognized response from Roku SoundBridge";
			return false;
		}

		return true;
	}
	
	// connects to the SoundBridge
	function connect($ip,$port = 4444,$timeout = 10) {
		$this->fp = fsockopen($ip,$port,$timeout);
		if (!$this->fp) {
			$this->error = "Could not connect to Roku SoundBridge";
			return false;
		}
		stream_set_blocking($this->fp, 0);
		usleep(100000);
		$this->_socket_clear();
		return true;
	}
		
	// disconnects from the SoundBridge
	function disconnect() {
		fwrite($this->fp,"exit\n");
		usleep(100000);
		fclose($this->fp);
		$this->fp = false;
	}
	
	// display a message on the SoundBridge's LCD for $duration microseconds
	function display($line1,$line2,$duration,$center = false) {
		if (!$this->fp) return false;
		
		fwrite($this->fp,"sketch\n");
		usleep(100000);
		
		$x1 = $x2 = 0;
		if ($center) {
			$screen_width = 40;
			
			$x1 = floor( ($screen_width - min(strlen($line1),$screen_width)) / 2);
			$x2 = floor( ($screen_width - min(strlen($line2),$screen_width)) / 2);
		}
		
		fwrite($this->fp,"text $x1 0 \"$line1\"\n");
		fwrite($this->fp,"text $x2 1 \"$line2\"\n");
		usleep($duration);
		fwrite($this->fp,"quit\n");
		usleep(100000);
		
		return true;
	}
		
	// return the SoundBridge uptime, in seconds
	function uptime() {
		$res = $this->_send_wait("uptime",100000);
		if (preg_match("/system uptime is ([0-9\.]+) sec/",$res,$matches)) {
			return $matches[1];
		} else {
			return false;
		}
	}
	
	// return an array containing the process list
	function process_list() {
		$res = $this->_send_wait("ps",false);
		$lines = explode("\n",trim($res));
		$column_def = str_replace(array("X S","D N"),array("X_S","D_N"),array_shift($lines));
		$cols = preg_split("/\s+/",trim($column_def));

		array_shift($lines);
		
		$processes = array();
		foreach ($lines as $line) {
			$data = preg_split("/\s+/",trim($line));
			$process = array();

			reset($cols);
			foreach ($cols as $k=>$col) {
				$process[$col] = $data[$k];
				if ($k==count($cols)-1) { 
					for ($i=$k+1; $i<count($data); $i++) {
						$process[$col] .= " ".$data[$i];
					}
				}
			}
			$processes[] = $process;
		}
		return $processes;
	}	
	
	// reboot the soundbridge
	function reboot() {
		fwrite($this->fp,"reboot\n");
		return true;
	}
	
	// $level is from 0 to 100, 100 being the brightest
	function lcd_brightness($level) {
		if (!is_numeric($level)) return false;
		$level = (int) $level;
		
		$this->_send_wait("lcdbr $level");
		return true;
	}
	
	// executes an arbitrary command and returns the response
	//
	// $command - specifies the command and any arguments to send
	// $oneline - returns after one response line is received if TRUE; returns when no more data is available if FALSE
	// $delay - specifies how long to wait before checking for a reply, in milliseconds
	function execute($command,$oneline=true,$delay=333333) {
		return $this->_send_wait($command,$oneline,$delay);
	}
	
	// list network interfaces in raw text format
	function network_interfaces() {
		return trim($this->_send_wait("ifconfig",false));
	}
	
	// returns detailed information about the Roku's network interface(s)
	//
	// returns an array with one element for each network interface; each element
	// is another array containing keys for the various network settings
	function network_interface_details() {
		$data = $this->network_interfaces();
		
		$interfaces = array();		
		$intfdata = array();
		
		if (preg_match_all('/Interface\: ([0-9]+)/i',$data,$matches,PREG_SET_ORDER)) $intfdata['id'] = $matches;
		if (preg_match_all('/has a (.*?) (?:assigned )?address/i',$data,$matches,PREG_SET_ORDER)) $intfdata['bootproto'] = $matches;
		if (preg_match_all('/MAC Address\: (.*?)$/im',$data,$matches,PREG_SET_ORDER)) $intfdata['mac'] = $matches;
		if (preg_match_all('/IP Address\: (.*?)$/im',$data,$matches,PREG_SET_ORDER)) $intfdata['ip'] = $matches;
		if (preg_match_all('/Netmask\: (.*?)$/im',$data,$matches,PREG_SET_ORDER)) $intfdata['netmask'] = $matches;
		if (preg_match_all('/Gateway\: (.*?)$/im',$data,$matches,PREG_SET_ORDER)) $intfdata['gateway'] = $matches;
		if (preg_match_all('/DNS 1\: (.*?)$/im',$data,$matches,PREG_SET_ORDER)) $intfdata['ns1'] = $matches;
		if (preg_match_all('/DNS 2\: (.*?)$/im',$data,$matches,PREG_SET_ORDER)) $intfdata['ns2'] = $matches;

		foreach ($intfdata as $name=>$intfvalues) {
			foreach ($intfvalues as $interfaceno=>$data) {
				$interfaces[$interfaceno][$name] = trim($data[1]);
			}
		}
		
		return $interfaces;
	}
	
	// returns an array of information about the currently playing song
	function get_current_song() {
		$songinfo = explode("\n",trim($this->_mmc_send_wait("GetCurrentSongInfo",false)));
		
		array_pop($songinfo);
		
		$output = array();
		foreach ($songinfo as $k=>$line) {
			list(,$key,$value) = explode(':',$line,3);
			$key = strtolower(trim($key));
			$value = trim($value);
			
			if (preg_match('/^(.*?)\[([0-9]+)\]\s+(.*?)$/',$key,$matches)) {
				$output[ $matches[1] ][ $matches[2] ][ $matches[3] ] = $value;
			} else {
				$output[$key] = $value;
			}
		}
		
		return $output;
	}
	
	// Play, Pause, or Stop audio playback
	function play() {	$this->_mmc_send('Play');	}
	function pause() {	$this->_mmc_send('Pause');	}
	function stop() {	$this->_mmc_send('Stop');	}
	
	// Navigate to next or previous song	
	function play_next()	{	$this->_mmc_send('Next'); 		}
	function play_prev()	{	$this->_mmc_send('Previous'); 	}
	
	// turns shuffle mode on or off
	function set_shuffle($on = false) {
		$newstate = $on ? 'on' : 'off';
		$this->_mmc_send_wait('Shuffle ' . $newstate);
	}

	// turns repeat mode on or off
	function set_repeat($on = false) {
		$newstate = $on ? 'on' : 'off';
		$this->_mmc_send_wait('Repeat ' . $newstate);
	}
	
	// obtains the elapsed time for this sessoin
	function get_elapsed_time() {
		$time = $this->_mmc_send_wait('GetElapsedTime');
		list(,$time) = explode(':',$time,2);
		return trim($time);
	}
	
	// obtains the current volume
	function get_volume() {
		$volume = $this->_mmc_send_wait('GetVolume');
		list(,$volume) = explode(':',$volume,2);
		return trim($volume);
	}

	// sets the volume
	function set_volume($level = 90) {
		$level = (int) $level;
		if ( ($level<0) || ($level>100) ) $level = 90;
		$this->_mmc_send_wait('SetVolume ' . $level);
	}
	
	
	// stop audio playback (instructs the Soundbridge directly rather than using the MMC console)
	function force_stop() {
		$this->_send_wait("stop");
		return true;
	}
	
	// plays back audio from an URL
	function htplay($url) {
		$this->_send_wait("htplay $url");
		return true;
	}
	
	// returns the SoundBridge event log
	function log() {
		return $this->_send_wait("log",false);
	}
	
	// clears the SoundBridge event log
	function clear_log() {
		$this->_send_wait("logclear");
		return true;
	}
	
	
	function _socket_clear() {
		while ( strlen($buffer = fread($this->fp,1024)) ) { }
	}
	
	function _socket_read($fullstring = true) {
		$res = "";
		$done = false;
		$i = 0;
		while (!$done) {
			if ($fullstring) {
				$data = fgets($this->fp,1024);
			} else {
				$data = fread($this->fp,1024);
			}
			
			$res .= $data;
			$done = ( strlen($res) && !strlen($data) );
			if ($fullstring) $done = (substr($res,-1)=="\n");
		}
		if (!$fullstring) {
			// remove the command prompt at the end
			return preg_replace('/\s*SoundBridge>\s*$/i','',$res);
		}
		
		return $res;
	}
	
	function _mmc_send_wait($cmd,$oneline=true,$delay=333333) {
		$this->_send_wait("mmc");
		$output = $this->_send_wait($cmd,$oneline,$delay);
		$this->_send_wait("exit");
		
		return $output;
	}
	
	function _mmc_send($cmd,$delay=333333) {

		$this->_send_wait("mmc");

		if (!$this->fp) return false;

		$this->_socket_clear();

		fwrite($this->fp,"$cmd\n");
		usleep($delay);

		$this->_send_wait("exit");
	}
	
	function _send_wait($cmd,$oneline=true,$delay=333333) {
		if (!$this->fp) return false;

		$this->_socket_clear();
	
		fwrite($this->fp,"$cmd\n");
		usleep($delay);
		
		$res = $this->_socket_read($oneline);
		return $res;
	}
	
}
?>
